
import { pino } from 'pino';
import { Req } from './http/request';

export interface LoggingConfig {
	level: 'silent' | 'fatal' | 'error' | 'warn' | 'info' | 'debug' | 'trace';
	pretty: boolean;
}

export function validate_logging_conf(conf: unknown) : asserts conf is LoggingConfig {
	// todo: validate config
}

export function create_logger(conf: LoggingConfig) {
	if (conf.pretty) {
		try {
			require('pino-pretty');
		} catch { }
	}

	return pino({
		level: conf.level,
		prettyPrint: conf.pretty && {
			translateTime: 'HH:MM:ss Z',
			ignore: 'pid,hostname'
		},
		serializers: {
			req(req: Req) {
				// Redact passwords from sharable private-entry URLs
				const req_url = req.url.replace(/\/([a-zA-Z0-9._-]+)\/(\d+)(\/?\?(?:.+&)?)?t=(?:[^&]+)/i, ($0, $1, $2, $3) => `/${$1}/${$2}${$3}t=[Redacted]`);
	
				return {
					method: req.method,
					url: req_url,
					hostname: req.hostname,
					remoteAddress: req.ip,
					remotePort: req.socket.remotePort,
					session: req.session?.key?.prefix,
				};
			}
		}
	});
}
