
import { FastifyReply } from 'fastify';
// import { redirect_302_found } from '../http';
import { BaseClient, Issuer, TypeOfGenericClient } from 'openid-client';
import { PKCECookieConfig } from './pkce-cookie';

const scopes = 'openid profile email';

export interface OIDCConfig {
	server_url: string;
	signing_algorithm: 'ES512';
	client_id: string;
	client_secret: string;
	pkce: PKCECookieConfig;
}

export function validate_oidc_conf(conf: unknown) : asserts conf is OIDCConfig {
	// todo: validate config
}

export function create_oidc_provider(conf: OIDCConfig) {
	let issuer: Issuer;
	let Client: TypeOfGenericClient<BaseClient>;
	let client: BaseClient;

	const ready = (async () => {
		issuer = await Issuer.discover(conf.server_url);
		Client = issuer.Client;
		client = new Client({
			client_id: conf.client_id,
			client_secret: conf.client_secret,
			id_token_signed_response_alg: conf.signing_algorithm,
			authorization_signed_response_alg: conf.signing_algorithm,
		});
	})();

	return {
		get ready() {
			return ready;
		},

		async redirect_to_auth_endpoint(res: FastifyReply, code_challenge: string, redirect_uri: string) {
			await ready;
	
			const uri = client.authorizationUrl({
				response_type: 'code',
				scope: scopes,
				redirect_uri,
				code_challenge,
				code_challenge_method: 'S256'
			});
		
			// todo: redirect
			// return redirect_302_found(res, uri, 'Logging in with OpenID Connect');
		}
	}
}
