
import { rand } from '../utilities/rand';
import { SessionData } from '../storage';
import { Argon2HashProvider } from './argon-hash';

export interface SessionKey {
	full_key: string;
	raw_key: string;
	prefix: string;
}

export type SessionKeyProvider = ReturnType<typeof create_session_key_provider>;

export function create_session_key_provider(argon2: Argon2HashProvider) {
	return {
		async generate() : Promise<SessionKey> {
			const bytes = await rand(48);
			const base64 = bytes.toString('base64url');
			const prefix = base64.slice(0, 16);
			const raw_key = base64.slice(16);
			const full_key = `${prefix}.${raw_key}`;
			
			return { prefix, raw_key, full_key };
		},
		parse(full_key: string) : SessionKey {
			const [ prefix, raw_key, ...rest ] = full_key.split('.');
		
			if (rest && rest.length) {
				throw new ParseSessionKeyError('Invalid session key');
			}
		
			return { prefix, raw_key, full_key };
		},
		verify(key: SessionKey, session: SessionData) : Promise<boolean> {
			return argon2.verify(key.raw_key, session.key_hash);
		},
	};
}

export class ParseSessionKeyError extends Error {
	public readonly name = 'ParseSessionKeyError';
}
